from vpython import *
import os
import numpy as np
import serial


def vec_mul(a, b):
    return vector(a._x * b._x, a._y * b._y, a._z * b._z)


def vec_inv(a):
    return vector(1 / a._x, 1 / a._y, 1 / a._z)


class CapacitorXYZ:
    def __init__(self, sername,
                 factor=(0.1, 0.1, 0.1),
                 offset=(0, 0, 0),
                 lookup_folder="lookup",
                 oversample=32):
        self.sercom = serial.Serial(sername, 115200)
        self.factor = vector(*factor)
        self.offset = vector(*offset)
        self.oversample = oversample
        self.lamb = 0.9
        self.lookup_folder = lookup_folder
        self.sercom.readline()
        self.pos = vector(0, 0, 0)
        self.lookup_xz = None
        self.lookup_yz = None
        self.lookup_xy = None
        self.read_lookup()
        self.calibrate()

    def read_lookup(self):
        self.lookup_xz = np.load(os.path.join(self.lookup_folder, "lookup_xz.npy"))
        self.lookup_yz = np.load(os.path.join(self.lookup_folder, "lookup_yz.npy"))
        self.lookup_xy = np.load(os.path.join(self.lookup_folder, "lookup_xy.npy"))

    def read_capa(self):
        line_str = self.sercom.readline().decode("ascii")
        v1, v2, v3, _ = line_str.split(" ")

        gen = (float(i) / self.oversample for i in (v2, v1, v3))

        vec = vector(*gen)

        return vec

    def capa_to_dist(self, capa):
        y = np.interp(capa._x, self.lookup_xz[:, 0], self.lookup_xz[:, 1])
        x = np.interp(capa._z, self.lookup_yz[:, 0], self.lookup_yz[:, 1])
        z = np.interp(capa._y, self.lookup_xy[:, 0], self.lookup_xy[:, 1])

        return vector(y, z, x)

    def read_pos(self):
        capa = self.read_capa()

        dist = self.capa_to_dist(capa)

        return vec_mul(dist, self.factor) + self.offset

    def update_pos(self):
        pos_new = self.read_pos()
        self.pos = self.pos * self.lamb + pos_new * (1 - self.lamb)

    def calibrate(self):
        self.pos = self.read_pos()

    def close(self):
        self.sercom.close()


def main():
    sercom = "COM74"

    scene.width = 800
    scene.height = 600
    scene.range = 1.3
    scene.title = "3D capacitor viewer\n"
    scene.ambient = color.gray(0.2)

    scene.lights = []

    distant_light(direction=vector(0.3, 0.6, 0.2), color=color.gray(0.8))

    running = True
    box_object = box(visible=True, size=vector(0.2, 0.2, 0.2))
    plane1 = box(visible=True, size=vector(3.5, 0.01, 3.5),
                 pos=vector(1.75, -0.005, 1.75),
                 texture={"file": "img/xy.png"})
    plane2 = box(visible=True, size=vector(3.5, 0.01, 2.5),
                 pos=vector(1.75, 1.25, -0.005),
                 up=vector(0, 0, 1),
                 texture={"file": "img/yz.png"})
    plane3 = box(visible=True, size=vector(2.5, 0.01, 3.5),
                 pos=vector(-0.005, 1.25, 1.75),
                 up=vector(1, 0, 0),
                 texture={"file": "img/xz.png"})

    curve_x = curve(vector(0, 0, 0), vector(0, 0, 0), color=color.green)
    curve_y = curve(vector(0, 0, 0), vector(0, 0, 0), color=color.blue)
    curve_z = curve(vector(0, 0, 0), vector(0, 0, 0), color=color.red)

    currentobject = box_object
    currentobject.visible = True

    cap = CapacitorXYZ(sercom)
    cap.calibrate()
    try:
        while True:
            if running:
                cap.update_pos()
                currentobject.pos = cap.pos
                curve_x.modify(0, pos=vector(0, cap.pos._y, cap.pos._z))
                curve_x.modify(1, pos=cap.pos)
                curve_y.modify(0, pos=vector(cap.pos._x, 0, cap.pos._z))
                curve_y.modify(1, pos=cap.pos)
                curve_z.modify(0, pos=vector(cap.pos._x, cap.pos._y, 0))
                curve_z.modify(1, pos=cap.pos)
    finally:
        cap.close()


if __name__ == "__main__":
    main()
